@@ -10,26 +10,25 @@
 // Just for this module
 #define _FILE_OFFSET_BITS 64
 
+#include "config.h"
 #include "bulk_extractor.h"
 #include "dig.h"
 
 #ifdef HAVE_LIBAFFLIB
-#ifndef HAVE_STL
-#define HAVE_STL			/* needed */
-#endif
-#include <afflib/afflib.h>
-#include <afflib/utils.h>
 #endif
 
 #include "image_process.h"
+#ifdef HAVE_LIBAFFLIB
+#ifndef HAVE_STL
+#define HAVE_STL			/* needed */
+#endif
+#include <afflib/utils.h>
+#endif
 
 #ifndef PATH_MAX
 #define PATH_MAX 65536
 #endif
 
-extern scanner_def scanners[];
-
-
 #ifndef MIN
 #define MIN(a,b) ((a)<(b) ? (a) : (b))
 #endif 
@@ -48,17 +47,35 @@ extern scanner_def scanners[];
  * files and devices. This seems to work. It only requires a functioning pread64 or pread.
  */
 
-#if !defined(HAVE_PREAD64) && !defined(HAVE_PREAD) && defined(HAVE__LSEEKI64)
+#ifdef WIN32
+int pread64(HANDLE current_handle,char *buf,size_t bytes,uint64_t offset)
+{
+    DWORD bytes_read = 0;
+    LARGE_INTEGER li;
+    li.QuadPart = offset;
+    li.LowPart = SetFilePointer(current_handle, li.LowPart, &li.HighPart, FILE_BEGIN);
+    if(li.LowPart == INVALID_SET_FILE_POINTER) return -1;
+    if (FALSE == ReadFile(current_handle, buf, (DWORD) bytes, &bytes_read, NULL)){
+        return -1;
+    }
+    return bytes_read;
+}
+#else
+  #if !defined(HAVE_PREAD64) && !defined(HAVE_PREAD) && defined(HAVE__LSEEKI64)
 static size_t pread64(int d,void *buf,size_t nbyte,int64_t offset)
 {
     if(_lseeki64(d,offset,0)!=offset) return -1;
     return read(d,buf,nbyte);
 }
+  #endif
 #endif
 
+#ifdef WIN32
+int64_t get_filesize(HANDLE fd)
+#else
 int64_t get_filesize(int fd)
+#endif
 {
-    struct stat st;
     char buf[64];
     int64_t raw_filesize = 0;		/* needs to be signed for lseek */
     int bits = 0;
@@ -77,10 +94,14 @@ int64_t get_filesize(int fd)
     }
 #endif
 
+#ifndef WIN32
     /* We can use fstat if sizeof(st_size)==8 and st_size>0 */
+    struct stat st;
+    memset(&st,0,sizeof(st));
     if(sizeof(st.st_size)==8 && fstat(fd,&st)==0){
 	if(st.st_size>0) return st.st_size;
     }
+#endif
 
     /* Phase 1; figure out how far we can seek... */
     for(bits=0;bits<60;bits++){
@@ -105,6 +126,33 @@ int64_t get_filesize(int fd)
     return raw_filesize;
 }
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
+static int64_t getSizeOfFile(const std::string &fname)
+{
+#ifdef WIN32
+    HANDLE current_handle = CreateFileA(fname.c_str(), FILE_READ_DATA,
+                                    FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
+				     OPEN_EXISTING, 0, NULL);
+    if(current_handle==INVALID_HANDLE_VALUE){
+        fprintf(stderr,"bulk_extractor WIN32 subsystem: cannot open file '%s'\n",fname.c_str());
+        return 0;
+    }
+    int64_t fname_length = get_filesize(current_handle);
+    ::CloseHandle(current_handle);
+#else
+    int fd = ::open(fname.c_str(),O_RDONLY|O_BINARY);
+    if(fd<0){
+        std::cerr << "*** unix getSizeOfFile: Cannot open " << fname << ": " << strerror(errno) << "\n";
+        return 0;
+    }
+    int64_t fname_length = get_filesize(fd);
+    ::close(fd);
+#endif
+    return fname_length;
+}
 
 
 
@@ -139,7 +187,7 @@ int process_aff::pread(unsigned char *buf,size_t bytes,int64_t offset) const
     return af_read(af,buf,bytes);
 }
 
-int64_t process_aff::image_size()
+int64_t process_aff::image_size() const
 {
     return af_get_imagesize(af);
 }
@@ -149,14 +197,14 @@ int64_t process_aff::image_size()
  * Iterator support
  */
 
-image_process::iterator process_aff::begin()
+image_process::iterator process_aff::begin() const
 {
     image_process::iterator it(this);
     it.raw_offset = 0;
     return it;
 }
 
-image_process::iterator process_aff::end()
+image_process::iterator process_aff::end() const
 {
     image_process::iterator it(this);
     it.page_counter = pagelist.size();
@@ -181,7 +229,7 @@ __END_DECLS
 /* Increment the AFF iterator by going to the next page.
  * If we hit the end of the pagelist, note that we are at the end of file.
  */
-void process_aff::increment_iterator(class image_process::iterator &it)
+void process_aff::increment_iterator(class image_process::iterator &it) const
 {
     if(it.page_counter < pagelist.size()){
 	it.page_counter++;
@@ -194,16 +242,14 @@ void process_aff::increment_iterator(class image_process::iterator &it)
 pos0_t process_aff::get_pos0(const image_process::iterator &it) const
 {
     int64_t pagenum = pagelist[it.page_counter];
-    pos0_t pos0;
-    pos0.offset = pagenum * af_get_pagesize(af);
-    return pos0;
+    return pos0_t("",pagenum * af_get_pagesize(af));
 }
 
-sbuf_t *process_aff::sbuf_alloc(image_process::iterator &it)
+sbuf_t *process_aff::sbuf_alloc(image_process::iterator &it) const
 {
     size_t bufsize  = af_get_pagesize(af)+margin;
     unsigned char *buf = (unsigned char *)malloc(bufsize);
-    if(!buf) throw bad_alloc();
+    if(!buf) throw std::bad_alloc();
 
     pos0_t pos0 = get_pos0(it);
     
@@ -224,24 +270,24 @@ sbuf_t *process_aff::sbuf_alloc(image_process::iterator &it)
     return 0;				// no buffer to return
 }
 
-double process_aff::fraction_done(class image_process::iterator &it)
+double process_aff::fraction_done(const image_process::iterator &it) const
 {
     return (double)it.page_counter / (double)pagelist.size();
 }
 
-string process_aff::str(class image_process::iterator &it)
+std::string process_aff::str(const image_process::iterator &it) const
 {
     char buf[64];
-    snprintf(buf,sizeof(buf),"Page %"PRId64"",it.page_counter);
-    return string(buf);
+    snprintf(buf,sizeof(buf),"Page %" PRId64 "",it.page_counter);
+    return std::string(buf);
 }
 
-uint64_t process_aff::blocks(class image_process::iterator &it)
+uint64_t process_aff::max_blocks(const image_process::iterator &it) const
 {
     return pagelist.size();
 }
 
-uint64_t process_aff::seek_block(class image_process::iterator &it,uint64_t block)
+uint64_t process_aff::seek_block(image_process::iterator &it,uint64_t block) const
 {
     it.page_counter = block;
     return block;
@@ -318,7 +364,6 @@ int process_ewf::open()
     }
     libewf_handle_get_media_size(handle,(size64_t *)&ewf_filesize,NULL);
 #else
-
     amount_of_filenames = libewf_glob(fname,strlen(fname),LIBEWF_FORMAT_UNKNOWN,&libewf_filenames);
     if(amount_of_filenames<0){
 	err(1,"libewf_glob");
@@ -363,22 +408,19 @@ int process_ewf::open()
     return 0;
 }
 
-vector<string> process_ewf::getewfdetails(){
-	return(details);
+std::vector<std::string> process_ewf::getewfdetails() const{
+    return(details);
 }
 
 
-int process_ewf::debug = 0;
+//int process_ewf::debug = 0;
 int process_ewf::pread(unsigned char *buf,size_t bytes,int64_t offset) const
 {
 #ifdef LIBEWFNG
     libewf_error_t *error=0;
     int ret = libewf_handle_read_random(handle,buf,bytes,offset,&error);
     if(ret<0){
-#ifdef HAVE_LIBEWF_ERROR_BACKTRACE_FPRINT
-	if(debug & DEBUG_PEDANTIC) libewf_error_backtrace_fprint(error,stderr);
-#endif
-	libewf_error_fprint(error,stderr);
+	if (report_read_errors) libewf_error_fprint(error,stderr);
 	libewf_error_free(&error);
     }
     return ret;
@@ -391,13 +433,13 @@ int process_ewf::pread(unsigned char *buf,size_t bytes,int64_t offset) const
 }
 
 
-int64_t process_ewf::image_size()
+int64_t process_ewf::image_size() const
 {
     return ewf_filesize;
 }
 
 
-image_process::iterator process_ewf::begin()
+image_process::iterator process_ewf::begin() const
 {
     image_process::iterator it(this);
     it.raw_offset = 0;
@@ -405,7 +447,7 @@ image_process::iterator process_ewf::begin()
 }
 
 
-image_process::iterator process_ewf::end()
+image_process::iterator process_ewf::end() const
 {
     image_process::iterator it(this);
     it.raw_offset = this->ewf_filesize;
@@ -415,13 +457,11 @@ image_process::iterator process_ewf::end()
 
 pos0_t process_ewf::get_pos0(const image_process::iterator &it) const
 {
-    pos0_t pos0;
-    pos0.offset = it.raw_offset;
-    return pos0;
+    return pos0_t("",it.raw_offset);
 }
 
 /** Read from the iterator into a newly allocated sbuf */
-sbuf_t *process_ewf::sbuf_alloc(image_process::iterator &it)
+sbuf_t *process_ewf::sbuf_alloc(image_process::iterator &it) const
 {
     int count = pagesize + margin;
 
@@ -430,7 +470,7 @@ sbuf_t *process_ewf::sbuf_alloc(image_process::iterator &it)
     }
 
     unsigned char *buf = (unsigned char *)malloc(count);
-    if(!buf) throw bad_alloc();			// no memory
+    if(!buf) throw std::bad_alloc();			// no memory
 
     count = this->pread(buf,count,it.raw_offset); // do the read
     if(count<0){
@@ -450,30 +490,30 @@ sbuf_t *process_ewf::sbuf_alloc(image_process::iterator &it)
 /**
  * just add the page size for process_ewf
  */
-void process_ewf::increment_iterator(image_process::iterator &it)
+void process_ewf::increment_iterator(image_process::iterator &it) const
 {
     it.raw_offset += pagesize;
     if(it.raw_offset > this->ewf_filesize) it.raw_offset = this->ewf_filesize;
 }
 
-double process_ewf::fraction_done(class image_process::iterator &it)
+double process_ewf::fraction_done(const image_process::iterator &it) const
 {
     return (double)it.raw_offset / (double)this->ewf_filesize;
 }
 
-string process_ewf::str(class image_process::iterator &it)
+std::string process_ewf::str(const image_process::iterator &it) const
 {
     char buf[64];
-    snprintf(buf,sizeof(buf),"Offset %"PRId64"MB",it.raw_offset/1000000);
-    return string(buf);
+    snprintf(buf,sizeof(buf),"Offset %" PRId64 "MB",it.raw_offset/1000000);
+    return std::string(buf);
 }
 
-uint64_t process_ewf::blocks(class image_process::iterator &it)
+uint64_t process_ewf::max_blocks(const image_process::iterator &it) const
 {
   return this->ewf_filesize / pagesize;
 }
 
-uint64_t process_ewf::seek_block(class image_process::iterator &it,uint64_t block)
+uint64_t process_ewf::seek_block(image_process::iterator &it,uint64_t block) const
 {
     it.raw_offset = pagesize * block;
     return block;
@@ -501,19 +541,20 @@ static bool fn_ends_with(const std::string &str,const std::string &suffix)
     return str.substr(str.size()-suffix.size())==suffix;
 }
 
-static bool is_multipart_file(string fn)
+static bool is_multipart_file(const std::string &fn)
 {
     return fn_ends_with(fn,".000")
 	|| fn_ends_with(fn,".001")
 	|| fn_ends_with(fn,"001.vmdk");
 }
 
-static string make_list_template(string fn,int *start)
+/* fn can't be & because it will get modified */
+static std::string make_list_template(std::string fn,int *start)
 {
     /* First find where the digits are */
     size_t p = fn.rfind("000");
-    if(p==string::npos) p = fn.rfind("001");
-    assert(p!=string::npos);
+    if(p==std::string::npos) p = fn.rfind("001");
+    assert(p!=std::string::npos);
     
     *start = atoi(fn.substr(p,3).c_str()) + 1;
     fn.replace(p,3,"%03d");	// make it a format
@@ -521,13 +562,13 @@ static string make_list_template(string fn,int *start)
 }
 
 
-process_raw::process_raw(string fname,size_t pagesize_,size_t margin_)
+process_raw::process_raw(const std::string &fname,size_t pagesize_,size_t margin_)
     :image_process(fname,pagesize_,margin_),
      file_list(),raw_filesize(0),current_file_name(),
 #ifdef WIN32
-                                        current_handle(INVALID_HANDLE_VALUE)
+     current_handle(INVALID_HANDLE_VALUE)
 #else
-                                        current_fd(-1)
+     current_fd(-1)
 #endif
 {
 }
@@ -540,23 +581,67 @@ process_raw::~process_raw() {
 #endif
 }
 
+#ifdef WIN32
+BOOL GetDriveGeometry(const wchar_t *wszPath, DISK_GEOMETRY *pdg)
+{
+    HANDLE hDevice = INVALID_HANDLE_VALUE;  // handle to the drive to be examined 
+    BOOL bResult   = FALSE;                 // results flag
+    DWORD junk     = 0;                     // discard results
+
+    hDevice = CreateFileW(wszPath,          // drive to open
+                          0,                // no access to the drive
+                          FILE_SHARE_READ | // share mode
+                          FILE_SHARE_WRITE, 
+                          NULL,             // default security attributes
+                          OPEN_EXISTING,    // disposition
+                          0,                // file attributes
+                          NULL);            // do not copy file attributes
+
+    if (hDevice == INVALID_HANDLE_VALUE)    // cannot open the drive
+        {
+            return (FALSE);
+        }
+
+    bResult = DeviceIoControl(hDevice,                       // device to be queried
+                              IOCTL_DISK_GET_DRIVE_GEOMETRY, // operation to perform
+                              NULL, 0,                       // no input buffer
+                              pdg, sizeof(*pdg),            // output buffer
+                              &junk,                         // # bytes returned
+                              (LPOVERLAPPED) NULL);          // synchronous I/O
+
+    CloseHandle(hDevice);
+
+    return (bResult);
+}
+
+#endif
+
 /**
  * Add the file to the list, keeping track of the total size
  */
-void process_raw::add_file(string fname)
+void process_raw::add_file(const std::string &fname)
 {
-    int fd = ::open(fname.c_str(),O_RDONLY|O_BINARY);
-    if(fd<0){
-	cerr << "*** Cannot open " << fname << ": " << strerror(errno) << "\n";
-	exit(1);
+    int64_t fname_length = 0;
+  
+    /* Get the physical size of drive under Windows */
+    fname_length = getSizeOfFile(fname);
+#ifdef WIN32
+    if (fname_length==0){
+        /* On Windows, see if we can use this */
+        fprintf(stderr,"%s checking physical drive\n",fname.c_str());
+        // http://msdn.microsoft.com/en-gb/library/windows/desktop/aa363147%28v=vs.85%29.aspx
+        DISK_GEOMETRY pdg = { 0 }; // disk drive geometry structure
+        std::wstring wszDrive = safe_utf8to16(fname);
+        GetDriveGeometry(wszDrive.c_str(), &pdg);
+        fname_length = pdg.Cylinders.QuadPart * (ULONG)pdg.TracksPerCylinder *
+            (ULONG)pdg.SectorsPerTrack * (ULONG)pdg.BytesPerSector;
     }
-    int64_t fname_length = get_filesize(fd);
-    ::close(fd);
+#endif
     file_list.push_back(file_info(fname,raw_filesize,fname_length));
     raw_filesize += fname_length;
 }
 
-const class process_raw::file_info *process_raw::find_offset(int64_t pos) const
+const process_raw::file_info *process_raw::find_offset(int64_t pos) const
 {
     for(process_raw::file_list_t::const_iterator it = file_list.begin();it != file_list.end();it++){
 	if((*it).offset<=pos && pos< ((*it).offset+(*it).length)){
@@ -576,18 +661,18 @@ int process_raw::open()
     /* Get the list of the files if this is a split-raw file */
     if(is_multipart_file(image_fname())){
 	int num=0;
-	string templ = make_list_template(image_fname(),&num);
+        std::string templ = make_list_template(image_fname(),&num);
 	for(;;num++){
 	    char probename[PATH_MAX];
 	    snprintf(probename,sizeof(probename),templ.c_str(),num); 
 	    if(access(probename,R_OK)!=0) break;	    // no more files
-	    add_file(string(probename)); // found another name
+	    add_file(std::string(probename)); // found another name
 	}
     }
     return 0;
 }
 
-int64_t process_raw::image_size()
+int64_t process_raw::image_size() const
 {
     return raw_filesize;
 }
@@ -600,20 +685,9 @@ int64_t process_raw::image_size()
  * 3. If there are additional files to read in the next file, recurse.
  */
 
-#if 0
-/* Create a pread64 if we are on Windows */
-#if !defined(HAVE_PREAD64) && !defined(HAVE_PREAD) && defined(HAVE__LSEEKI64)
-static size_t pread64(int d,void *buf,size_t nbyte,int64_t offset)
-{
-    if(_lseeki64(d,offset,0)!=offset) return -1;
-    return read(d,buf,nbyte);
-}
-#endif
-#endif
-
 int process_raw::pread(unsigned char *buf,size_t bytes,int64_t offset) const
 {
-    const class file_info *fi = find_offset(offset);
+    const file_info *fi = find_offset(offset);
     if(fi==0) return 0;			// nothing to read.
 
     /* See if the file is the one that's currently opened.
@@ -628,10 +702,15 @@ int process_raw::pread(unsigned char *buf,size_t bytes,int64_t offset) const
 #endif
 
 	current_file_name = fi->name;
+	fprintf(stderr,"Attempt to open %s\n",fi->name.c_str());
 #ifdef WIN32
         current_handle = CreateFileA(fi->name.c_str(), FILE_READ_DATA,
-                                    FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
-        if(current_handle==INVALID_HANDLE_VALUE) return -1;
+                                    FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
+				     OPEN_EXISTING, 0, NULL);
+        if(current_handle==INVALID_HANDLE_VALUE){
+	  fprintf(stderr,"bulk_extractor WIN32 subsystem: cannot open file '%s'\n",fi->name.c_str());
+	  return -1;
+	}
 #else        
 	current_fd = ::open(fi->name.c_str(),O_RDONLY|O_BINARY);
 	if(current_fd<=0) return -1;	// can't read this data
@@ -654,7 +733,7 @@ int process_raw::pread(unsigned char *buf,size_t bytes,int64_t offset) const
 #ifdef WIN32
     DWORD bytes_read = 0;
     LARGE_INTEGER li;
-    li.QuadPart = offset;
+    li.QuadPart = offset - fi->offset;
     li.LowPart = SetFilePointer(current_handle, li.LowPart, &li.HighPart, FILE_BEGIN);
     if(li.LowPart == INVALID_SET_FILE_POINTER) return -1;
     if (FALSE == ReadFile(current_handle, buf, (DWORD) bytes, &bytes_read, NULL)){
@@ -675,7 +754,7 @@ int process_raw::pread(unsigned char *buf,size_t bytes,int64_t offset) const
 }
 
 
-image_process::iterator process_raw::begin()
+image_process::iterator process_raw::begin() const
 {
     image_process::iterator it(this);
     return it;
@@ -683,7 +762,7 @@ image_process::iterator process_raw::begin()
 
 
 /* Returns an iterator at the end of the image */
-image_process::iterator process_raw::end()
+image_process::iterator process_raw::end() const
 {
     image_process::iterator it(this);
     it.raw_offset = this->raw_filesize;
@@ -691,37 +770,34 @@ image_process::iterator process_raw::end()
     return it;
 }
 
-
-void process_raw::increment_iterator(image_process::iterator &it)
+void process_raw::increment_iterator(image_process::iterator &it) const
 {
     it.raw_offset += pagesize;
     if(it.raw_offset > this->raw_filesize) it.raw_offset = this->raw_filesize;
 }
 
-double process_raw::fraction_done(class image_process::iterator &it)
+double process_raw::fraction_done(const image_process::iterator &it) const
 {
     return (double)it.raw_offset / (double)this->raw_filesize;
 }
 
-string process_raw::str(class image_process::iterator &it)
+std::string process_raw::str(const image_process::iterator &it) const
 {
     char buf[64];
-    snprintf(buf,sizeof(buf),"Offset %"PRId64"MB",it.raw_offset/1000000);
-    return string(buf);
+    snprintf(buf,sizeof(buf),"Offset %" PRId64 "MB",it.raw_offset/1000000);
+    return std::string(buf);
 }
 
 
 pos0_t process_raw::get_pos0(const image_process::iterator &it) const
 {
-    pos0_t pos0;
-    pos0.offset = it.raw_offset;
-    return pos0;
+    return pos0_t("",it.raw_offset);
 }
 
 /** Read from the iterator into a newly allocated sbuf.
  * uses pagesize.
  */
-sbuf_t *process_raw::sbuf_alloc(image_process::iterator &it)
+sbuf_t *process_raw::sbuf_alloc(image_process::iterator &it) const
 {
     int count = pagesize + margin;
 
@@ -729,8 +805,8 @@ sbuf_t *process_raw::sbuf_alloc(image_process::iterator &it)
 	count = this->raw_filesize - it.raw_offset;
     }
     unsigned char *buf = (unsigned char *)malloc(count);
-    if(!buf) throw bad_alloc();			// no memory
-    count = this->pread(buf,count,it.raw_offset); // do the read
+    if(!buf) throw std::bad_alloc();			// no memory
+    count = this->pread(buf,count,it.raw_offset);       // do the read
     if(count==0){
 	free(buf);
 	it.eof = true;
@@ -751,12 +827,12 @@ static std::string filename_extension(std::string fn)
     return fn.substr(dotpos+1);
 }
 
-uint64_t process_raw::blocks(class image_process::iterator &it)
+uint64_t process_raw::max_blocks(const image_process::iterator &it) const
 {
     return (this->raw_filesize+pagesize-1) / pagesize;
 }
 
-uint64_t process_raw::seek_block(class image_process::iterator &it,uint64_t block)
+uint64_t process_raw::seek_block(image_process::iterator &it,uint64_t block) const
 {
     if(block * pagesize > (uint64_t)raw_filesize){
         block = raw_filesize / pagesize;
@@ -767,8 +843,6 @@ uint64_t process_raw::seek_block(class image_process::iterator &it,uint64_t bloc
 }
 
 
-
-
 /****************************************************************
  *** Directory Recursion
  ****************************************************************/
@@ -807,18 +881,18 @@ int process_dir::pread(unsigned char *buf,size_t bytes,int64_t offset) const
     err(1,"process_dir does not support pread");
 }
 
-int64_t process_dir::image_size()
+int64_t process_dir::image_size() const
 {
     return files.size();		// the 'size' is in files
 }
 
-image_process::iterator process_dir::begin()
+image_process::iterator process_dir::begin() const
 {
     image_process::iterator it(this);
     return it;
 }
 
-image_process::iterator process_dir::end()
+image_process::iterator process_dir::end() const
 {
     image_process::iterator it(this);
     it.file_number = files.size();
@@ -826,7 +900,7 @@ image_process::iterator process_dir::end()
     return it;
 }
 
-void process_dir::increment_iterator(image_process::iterator &it)
+void process_dir::increment_iterator(image_process::iterator &it) const
 {
     it.file_number++;
     if(it.file_number>files.size()) it.file_number=files.size();
@@ -834,40 +908,37 @@ void process_dir::increment_iterator(image_process::iterator &it)
 
 pos0_t process_dir::get_pos0(const image_process::iterator &it) const
 {
-    pos0_t pos0;
-    pos0.path = files[it.file_number];
-    pos0.offset = 0;
-    return pos0;
+    return pos0_t(files[it.file_number],0);
 }
 
 /** Read from the iterator into a newly allocated sbuf
  * with mapped memory.
  */
-sbuf_t *process_dir::sbuf_alloc(image_process::iterator &it)
+sbuf_t *process_dir::sbuf_alloc(image_process::iterator &it) const
 {
-    std::string fn = files[it.file_number];
-    sbuf_t *sbuf = sbuf_t::map_file(fn,pos0_t(fn+sbuf_t::U10001C));
+    std::string fname = files[it.file_number];
+    sbuf_t *sbuf = sbuf_t::map_file(fname);
     if(sbuf==0) throw read_error();	// can't read
     return sbuf;
 }
 
-double process_dir::fraction_done(class image_process::iterator &it)
+double process_dir::fraction_done(const image_process::iterator &it) const
 {
     return (double)it.file_number / (double)files.size();
 }
 
-string process_dir::str(class image_process::iterator &it)
+std::string process_dir::str(const image_process::iterator &it) const
 {
-    return string("File ")+files[it.file_number];
+    return std::string("File ")+files[it.file_number];
 }
 
 
-uint64_t process_dir::blocks(class image_process::iterator &it)
+uint64_t process_dir::max_blocks(const image_process::iterator &it) const
 {
     return files.size();
 }
 
-uint64_t process_dir::seek_block(class image_process::iterator &it,uint64_t block)
+uint64_t process_dir::seek_block(class image_process::iterator &it,uint64_t block) const
 {
     it.file_number = block;
     return it.file_number;
@@ -882,14 +953,20 @@ uint64_t process_dir::seek_block(class image_process::iterator &it,uint64_t bloc
 
 #include <functional>
 #include <locale>
-image_process *image_process::open(string fn,bool opt_recurse,
+image_process *image_process::open(std::string fn,bool opt_recurse,
                                    size_t pagesize_,size_t margin_)
 {
     image_process *ip = 0;
-    string ext = filename_extension(fn);
+    std::string ext = filename_extension(fn);
     struct stat st;
+    bool  is_windows_unc = false;
 
-    if(stat(fn.c_str(),&st)){
+#ifdef WIN32
+    if(fn.size()>2 && fn[0]=='\\' && fn[1]=='\\') is_windows_unc=true;
+#endif
+
+    memset(&st,0,sizeof(st));
+    if(stat(fn.c_str(),&st) && !is_windows_unc){
 	return 0;			// no file?
     }
     if(S_ISDIR(st.st_mode)){
@@ -938,15 +1015,15 @@ image_process *image_process::open(string fn,bool opt_recurse,
 #ifdef HAVE_LIBAFFLIB
 	    ip = new process_aff(fn,pagesize_,margin_);
 #else
-	    cerr << "This program was compiled without AFF support\n";
-	    exit(0);
+            std::cerr << "This program was compiled without AFF support\n";
+	    exit(1);
 #endif
 	}
 	if(ext=="e01"){
 #ifdef HAVE_LIBEWF
 	    ip = new process_ewf(fn,pagesize_,margin_);
 #else
-	    cerr << "This program was compiled without E01 support\n";
+            std::cerr << "This program was compiled without E01 support\n";
 	    exit(1);
 #endif
 	}
@@ -954,7 +1031,8 @@ image_process *image_process::open(string fn,bool opt_recurse,
     }
     /* Try to open it */
     if(ip->open()){
-	errx(1,"Cannot open %s",fn.c_str());
+	fprintf(stderr,"Cannot open %s",fn.c_str());
+        return 0;
     }
     return ip;
 }
