@@ -27,7 +27,7 @@
 #include "be13_api/utils.h"
 
 #include <set>
-#include <tr1/unordered_set>
+//#include <tr1/unordered_set>
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -231,7 +231,7 @@ static uint16_t IPv6L3Chksum(const sbuf_t &sbuf, u_int chksum_byteoffset)
     const struct ip6_hdr *ip6 = sbuf.get_struct_ptr<struct ip6_hdr>(0);
     if(ip6==0) return 0;		// cannot compute; not enough data
 	
-    int len = ntohs(ip6->ip6_plen) + 40;/* payload len + size of pseudo hdr */
+    int len      = ntohs(ip6->ip6_plen) + 40;/* payload len + size of pseudo hdr */
     uint32_t sum = 0;			// 
     u_int octets_processed = 0;
 
@@ -342,15 +342,18 @@ static bool invalidIP4(const uint8_t *const cc)
     if ( (cc[0] == 0) || (cc[0] == 255) ){
         return true;
     }
+
     /* IANA Reserved http://www.iana.org/assignments/ipv4-address-space/ipv4-address-space.txt */
     if (cc[0] == 127) return true;
     if (cc[0] >= 224) return true;
+
     /* Sequences of middle 0x0000 or 0xffff */
     if ( (cc[1] == 0)   && (cc[2] == 0) ) return true;
     if ( (cc[1] == 255) && (cc[2] == 255) ) return true;
+
     /* Trailing zero or 0xff */
     if ( (cc[3] == 0) || (cc[3] == 255) ) return true;
-    if( (cc[0]==cc[1]) && (cc[1]==cc[2]) ) return true;    /* Palendromes; needed? */
+    if ( (cc[0]==cc[1]) && (cc[1]==cc[2]) ) return true;    /* Palendromes; needed? */
     return false;
 }
 
@@ -382,7 +385,7 @@ static bool invalidIP(const uint8_t addr[16], sa_family_t family) {
     }
 }
 
-static string ip2string(const struct be13::ip4_addr *const a)
+static std::string ip2string(const struct be13::ip4_addr *const a)
 {
     const uint8_t *b = (const uint8_t *)a;
 
@@ -403,29 +406,28 @@ const char *inet_ntop(int af, const void *src, char *dst, size_t size)
 #endif
 
 
-static string ip2string(const uint8_t *addr, sa_family_t family)
+static std::string ip2string(const uint8_t *addr, sa_family_t family)
 {
     char printstr[INET6_ADDRSTRLEN+1];
     switch (family) {
     case AF_INET: 
-	return string(inet_ntop(family,reinterpret_cast<const struct in_addr *>(addr+12), printstr, sizeof(printstr)));
+	return std::string(inet_ntop(family,reinterpret_cast<const struct in_addr *>(addr+12), printstr, sizeof(printstr)));
     case AF_INET6: 
-	return string(inet_ntop(family, addr, printstr, sizeof(printstr)));
+	return std::string(inet_ntop(family, addr, printstr, sizeof(printstr)));
     }
-    return string("INVALID family ");
+    return std::string("INVALID family ");
 }
 
-static string mac2string(const struct be13::ether_addr *const e)
+static std::string mac2string(const struct be13::ether_addr *const e)
 {
     char addr[32];
     snprintf(addr,sizeof(addr),"%02X:%02X:%02X:%02X:%02X:%02X",
 	     e->ether_addr_octet[0], e->ether_addr_octet[1], e->ether_addr_octet[2],
 	     e->ether_addr_octet[3], e->ether_addr_octet[4], e->ether_addr_octet[5]);
-    string res(addr);
-    return res;
+    return std::string(addr);
 }
 
-static string i2str(const int i)
+static std::string i2str(const int i)
 {
     std::stringstream  ss;
     ss << i;
@@ -459,7 +461,7 @@ static bool sanePort(const uint16_t port) {
 static bool sanityCheckIP46Header(const sbuf_t &sbuf, bool *checksum_valid, generic_iphdr_t *h)
 {
     const struct be13::ip4 *ip = sbuf.get_struct_ptr<struct be13::ip4>(0);
-    if(!ip) return false;		// not enough space
+    if (!ip) return false;		// not enough space
     if (ip->ip_v == 4){
 	if (ip->ip_hl != 5) return false;	// IPv4 header length is 20 bytes (5 quads) (ignores options)
 	if ( (ip->ip_off != 0x0) && (ip->ip_off != ntohs(IP_DF)) ) return false;
@@ -490,7 +492,7 @@ static bool sanityCheckIP46Header(const sbuf_t &sbuf, bool *checksum_valid, gene
     } 
 
     const struct be13::ip6_hdr *ip6 = sbuf.get_struct_ptr<struct be13::ip6_hdr>(0);
-    if(!ip6) return false;
+    if (!ip6) return false;
     if ((ip6->ip6_vfc & 0xF0) == 0x60){
 		
 	//only do TCP, UDP and ICMPv6
@@ -597,21 +599,21 @@ static bool   likely_valid_pcap_header(const sbuf_t &sbuf,struct pcap_hdr &h)
  */
 class packet_carver {
 private:
-    packet_carver(const packet_carver &pc):fs(pc.fs),ps(pc.ps),ip_recorder(pc.ip_recorder),tcp_recorder(pc.tcp_recorder),ether_recorder(pc.ether_recorder){
+    packet_carver(const packet_carver &pc):fs(pc.fs),/*ps(pc.ps),*/ip_recorder(pc.ip_recorder),tcp_recorder(pc.tcp_recorder),ether_recorder(pc.ether_recorder){
     }
     packet_carver &operator=(const packet_carver &that){
 	return *this;			// no-op
     }
 public:
-    typedef std::tr1::unordered_set<const void *> packetset;
+    //typedef std::tr1::unordered_set<const void *> packetset;
     feature_recorder_set &fs;
-    packetset ps;
+    //packetset ps;
     feature_recorder *ip_recorder;
     feature_recorder *tcp_recorder;
     feature_recorder *ether_recorder;
 
     packet_carver(const class scanner_params &sp):
-	fs(sp.fs),ps(),ip_recorder(0),tcp_recorder(0),ether_recorder(0){
+	fs(sp.fs),/*ps(),*/ip_recorder(0),tcp_recorder(0),ether_recorder(0){
 	ip_recorder = fs.get_name("ip");
 	ether_recorder = fs.get_name("ether");
 	if(carve_net_memory){
@@ -656,7 +658,7 @@ public:
 	// Make sure that neither this packet nor an encapsulated version of this packet has been written
 	cppmutex::lock lock(Mfcap);		// lock the mutex
         if(fcap==0){
-            string ofn = ip_recorder->outdir+"/" + default_filename;
+            std::string ofn = ip_recorder->get_outdir() + "/" + default_filename;
             fcap = fopen(ofn.c_str(),"wb"); // write the output
             pcap_write4(0xa1b2c3d4);
             pcap_write2(2);			// major version number
@@ -760,8 +762,8 @@ public:
      * Please remember this is called for every byte of the disk image,
      * so it needs to be as fast as possible.
      */
-    static string chksum_ok;
-    static string chksum_bad;
+    static std::string chksum_ok;
+    static std::string chksum_bad;
 
     /** Write the ethernet addresses and the TCP info into the appropriate feature files.
      */
@@ -769,8 +771,8 @@ public:
     void documentIPFields(const sbuf_t &sb2,const generic_iphdr_t &h,bool checksum_valid) {
 	/* Report the IP address */
 	/* based on the TTL, infer whether remote or local */
-	const string &chksum_status = checksum_valid ? chksum_ok : chksum_bad;
-	string src,dst;
+	const std::string &chksum_status = checksum_valid ? chksum_ok : chksum_bad;
+        std::string src,dst;
 		    
 	if(isPowerOfTwo(h.ttl)){
 	    src = "L";	// src is local because the power of two hasn't been decremented
@@ -781,7 +783,7 @@ public:
 	}
 	
 	/* Record the IP addresses */
-	const string name = (h.family==AF_INET) ? "ip " : "ip6_hdr ";
+	const std::string name = (h.family==AF_INET) ? "ip " : "ip6_hdr ";
 	ip_recorder->write(sb2.pos0, ip2string(h.src, h.family),
 			   "struct " + name + src + " (src) " + chksum_status);
 	ip_recorder->write(sb2.pos0, ip2string(h.dst, h.family),
@@ -1001,7 +1003,7 @@ public:
 	    }
 	    if(carved==0 && carve_net_memory){
 		/* If we can't carve a packet, look for these two memory structures */
-		carved = max(carveSockAddrIn( sb2 ), carveTCPTOBJ( sb2 ));
+		carved = std::max(carveSockAddrIn( sb2 ), carveTCPTOBJ( sb2 ));
 	    }
 	    i += (carved>0 ? carved : 1);	// advance the pointer
 	}
@@ -1010,8 +1012,8 @@ public:
     };
 };
 
-string packet_carver::chksum_ok("cksum-ok");
-string packet_carver::chksum_bad("cksum-bad");
+std::string packet_carver::chksum_ok("cksum-ok");
+std::string packet_carver::chksum_bad("cksum-bad");
 
 extern "C"
 void scan_net(const class scanner_params &sp,const recursion_control_block &rcb)
