@@ -36,8 +36,8 @@
 /* fat32 tuning parameters for weirdness. Each of these define something weird. If too much is weird, it's probably not a FAT32 directory entry.. */
 static uint32_t opt_weird_file_size    = 1024*1024*150; // max file size
 static uint32_t opt_weird_file_size2   = 1024*1024*512; // max file size
-static uint32_t opt_max_cluster    = 32*CLUSTERS_IN_1GiB; // assume smaller than 32GB with 512 byte clusters
-static uint32_t opt_max_cluster2   = 128*CLUSTERS_IN_1GiB; // assume smaller than 512GB with 512 byte clusters
+static uint32_t opt_weird_cluster_count    = 32*CLUSTERS_IN_1GiB; // assume smaller than 32GB with 512 byte clusters
+static uint32_t opt_weird_cluster_count2   = 128*CLUSTERS_IN_1GiB; // assume smaller than 512GB with 512 byte clusters
 static uint32_t opt_max_bits_in_attrib = 3;
 static uint32_t opt_max_weird_count    = 2;
 static uint32_t opt_last_year = 2020;
@@ -185,15 +185,6 @@ fat_validation_t valid_fat_directory_entry(const sbuf_t &sbuf)
 	    return INVALID;			// can't have both DIRECTORY and ARCHIVE set
 	}
 
-#if 0
-        /* This is for debugging specific filename bugs */
-        bool dn=false;
-        if(strncmp((const char *)dentry.name,"SYSLINUX",8)==0 && strncmp((const char *)dentry.ext,"CFG",3)==0){
-            dn=true;
-            printf("dn=%d\n",dn);
-        }
-#endif
-
         if(dentry.attrib & 0x40) return INVALID; // "Device, never found on disk" (wikipedia)
 
 	if(!valid_fat_dentry_name(dentry.name,dentry.ext)) return INVALID; // invalid name
@@ -228,14 +219,12 @@ fat_validation_t valid_fat_directory_entry(const sbuf_t &sbuf)
         if(fat32int(dentry.size) > opt_weird_file_size) weird_count++;
         if(fat32int(dentry.size) > opt_weird_file_size2) weird_count++;
         if(count_bits(dentry.attrib) > opt_max_bits_in_attrib) weird_count++;
-        if(fat32int(dentry.highclust,dentry.startclust) > opt_max_cluster) weird_count++;
-        if(fat32int(dentry.highclust,dentry.startclust) > opt_max_cluster2) weird_count++;
+        if(fat32int(dentry.highclust,dentry.startclust) > opt_weird_cluster_count) weird_count++;
+        if(fat32int(dentry.highclust,dentry.startclust) > opt_weird_cluster_count2) weird_count++;
         if(dentry.ctimeten != 0 && dentry.ctimeten != 100) weird_count++;
         if(adate==0 && cdate==0) weird_count++;
         if(adate==0 && wdate==0) weird_count++;
 
-        //printf("wc=%d \n",weird_count);
-
         if(weird_count > opt_max_weird_count) return INVALID;
                                                                            
     }
@@ -388,11 +377,6 @@ void scan_ntfsdirs(const sbuf_t &sbuf,feature_recorder *wrecorder)
 			    continue;
 			}
 
-			if(attr_type==NTFS_ATYPE_SI){
-			    found_attrs++;
-			    if(debug & DEBUG_INFO) std::cerr << "NTFS_ATYPE_SI ignored\n";
-			}
-		    
 			if(attr_type==NTFS_ATYPE_ATTRLIST){
 			    found_attrs++;
 			    if(debug & DEBUG_INFO) std::cerr << "NTFS_ATTRLIST ignored\n";
@@ -415,10 +399,10 @@ void scan_ntfsdirs(const sbuf_t &sbuf,feature_recorder *wrecorder)
 						     | ((uint64_t)n[attr_off+soff+4]<<32)
 						     | ((uint64_t)n[attr_off+soff+5]<<40));
 			    mftmap["par_seq"] = utos(n.get16u(attr_off+soff+6));
-			    mftmap["crtime"] = microsoftDateToISODate(n.get64u(attr_off+soff+8));
-			    mftmap["mtime"]  = microsoftDateToISODate(n.get64u(attr_off+soff+16));
-			    mftmap["ctime"]  = microsoftDateToISODate(n.get64u(attr_off+soff+24));
-			    mftmap["atime"]  = microsoftDateToISODate(n.get64u(attr_off+soff+32));
+			    mftmap["crtime_fn"] = microsoftDateToISODate(n.get64u(attr_off+soff+8));
+			    mftmap["mtime_fn"]  = microsoftDateToISODate(n.get64u(attr_off+soff+16));
+			    mftmap["ctime_fn"]  = microsoftDateToISODate(n.get64u(attr_off+soff+24));
+			    mftmap["atime_fn"]  = microsoftDateToISODate(n.get64u(attr_off+soff+32));
 
 			    // these can be sanity checked
 
@@ -450,6 +434,66 @@ void scan_ntfsdirs(const sbuf_t &sbuf,feature_recorder *wrecorder)
 			    filename = safe_utf16to8(utf16str);
 			    mftmap["filename"] = filename;
 			}
+
+			if(attr_type==NTFS_ATYPE_SI){
+			    found_attrs++;
+			    if(debug & DEBUG_INFO) std::cerr << "NTFS_ATYPE_SI\n";
+
+			    size_t soff         = n.get16u(attr_off+20);
+
+			    mftmap["crtime_si"] = microsoftDateToISODate(n.get64u(attr_off+soff+0));
+			    mftmap["mtime_si"]  = microsoftDateToISODate(n.get64u(attr_off+soff+8));
+			    mftmap["ctime_si"]  = microsoftDateToISODate(n.get64u(attr_off+soff+16));
+			    mftmap["atime_si"]  = microsoftDateToISODate(n.get64u(attr_off+soff+24));
+			}
+
+                        if(attr_type==NTFS_ATYPE_OBJID){
+                            found_attrs++;
+                            if(debug & DEBUG_INFO) std::cerr << "NTFS_ATYPE_OBJID\n";
+
+                            size_t slen         = n.get32u(attr_off+16);
+                            size_t soff         = n.get16u(attr_off+20);
+                            char guid_objid[37], guid_bvolid[37], guid_bobjid[37], guid_domid[37];
+
+			    if(debug & DEBUG_INFO) std::cerr << " soff=" << soff << " slen=" << slen
+							     << "\n";    
+
+                            if(slen>=16){
+                                snprintf(guid_objid, 37, "%02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%02x%02x%02x%02x%02x%02x", \
+                                    n[attr_off+soff+3], n[attr_off+soff+2], n[attr_off+soff+1], n[attr_off+soff+0],      \
+                                    n[attr_off+soff+5], n[attr_off+soff+4], n[attr_off+soff+7], n[attr_off+soff+6],      \
+                                    n[attr_off+soff+8], n[attr_off+soff+9], n[attr_off+soff+10], n[attr_off+soff+11],    \
+                                    n[attr_off+soff+12], n[attr_off+soff+13], n[attr_off+soff+14], n[attr_off+soff+15]);
+                                mftmap["guid_objectid"] = guid_objid;
+                            }
+                            if(slen>=32){
+                                soff+=16;
+                                snprintf(guid_bvolid, 37, "%02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%02x%02x%02x%02x%02x%02x", \
+                                    n[attr_off+soff+3], n[attr_off+soff+2], n[attr_off+soff+1], n[attr_off+soff+0],      \
+                                    n[attr_off+soff+5], n[attr_off+soff+4], n[attr_off+soff+7], n[attr_off+soff+6],      \
+                                    n[attr_off+soff+8], n[attr_off+soff+9], n[attr_off+soff+10], n[attr_off+soff+11],    \
+                                    n[attr_off+soff+12], n[attr_off+soff+13], n[attr_off+soff+14], n[attr_off+soff+15]);
+                                mftmap["guid_birthvolumeid"] = guid_bvolid;
+                            }
+                            if(slen>=48){
+                                soff+=16;
+                                snprintf(guid_bobjid, 37, "%02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%02x%02x%02x%02x%02x%02x", \
+                                    n[attr_off+soff+3], n[attr_off+soff+2], n[attr_off+soff+1], n[attr_off+soff+0],      \
+                                    n[attr_off+soff+5], n[attr_off+soff+4], n[attr_off+soff+7], n[attr_off+soff+6],      \
+                                    n[attr_off+soff+8], n[attr_off+soff+9], n[attr_off+soff+10], n[attr_off+soff+11],    \
+                                    n[attr_off+soff+12], n[attr_off+soff+13], n[attr_off+soff+14], n[attr_off+soff+15]);
+                                mftmap["guid_birthobjectid"] = guid_bobjid;
+                            }
+                            if(slen>=64){
+                                soff+=16;
+                                snprintf(guid_domid, 37, "%02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%02x%02x%02x%02x%02x%02x", \
+                                    n[attr_off+soff+3], n[attr_off+soff+2], n[attr_off+soff+1], n[attr_off+soff+0],      \
+                                    n[attr_off+soff+5], n[attr_off+soff+4], n[attr_off+soff+7], n[attr_off+soff+6],      \
+                                    n[attr_off+soff+8], n[attr_off+soff+9], n[attr_off+soff+10], n[attr_off+soff+11],    \
+                                    n[attr_off+soff+12], n[attr_off+soff+13], n[attr_off+soff+14], n[attr_off+soff+15]);
+                                mftmap["guid_domainid"] = guid_domid;
+                            }
+                        }
 		    
 			attr_off += attr_len;
 		    }		
@@ -476,24 +520,32 @@ void scan_ntfsdirs(const sbuf_t &sbuf,feature_recorder *wrecorder)
 extern "C"
 void scan_windirs(const class scanner_params &sp,const recursion_control_block &rcb)
 {
-    string myString;
+    std::string myString;
     assert(sp.sp_version==scanner_params::CURRENT_SP_VERSION);
     if(sp.phase==scanner_params::PHASE_STARTUP){
+
+        /* Figure out the current time */
+        time_t t = time(0);
+        struct tm now;
+        memset(&now,0,sizeof(now));     // assures all of now is cleared; required for static analysis tools
+        gmtime_r(&t,&now);                     
+        opt_last_year = now.tm_year + 1900 + 5; // allow up to 5 years in the future
+
         assert(sp.info->si_version==scanner_info::CURRENT_SI_VERSION);
 	sp.info->name		= "windirs";
-        sp.info->author         = "Simson Garfinkel";
+        sp.info->author         = "Simson Garfinkel and Maxim Suhanov";
         sp.info->description    = "Scans Microsoft directory structures";
 	sp.info->flags =  scanner_info::SCANNER_DEPTH_0; // only run at top level by default
         sp.info->scanner_version= "1.0";
 	sp.info->feature_names.insert("windirs");
 
-        sp.info->get_config("opt_weird_file_size",&opt_weird_file_size,"Weird file size");
-        sp.info->get_config("opt_weird_file_size2",&opt_weird_file_size2,"Weird file size2");
-        sp.info->get_config("opt_max_cluster",&opt_max_cluster,"Ignore clusters larger than this");
-        sp.info->get_config("opt_max_cluster2",&opt_max_cluster2,"Ignore clusters larger than this");
+        sp.info->get_config("opt_weird_file_size",&opt_weird_file_size,"Threshold for FAT32 scanner");
+        sp.info->get_config("opt_weird_file_size2",&opt_weird_file_size2,"Threshold for FAT32 scanner");
+        sp.info->get_config("opt_weird_cluster_count",&opt_weird_cluster_count,"Threshold for FAT32 scanner");
+        sp.info->get_config("opt_weird_cluster_count2",&opt_weird_cluster_count2,"Threshold for FAT32 scanner");
         sp.info->get_config("opt_max_bits_in_attrib",&opt_max_bits_in_attrib,
                             "Ignore FAT32 entries with more attributes set than this");
-        sp.info->get_config("opt_max_weird_count",&opt_max_weird_count,"Ignore FAT32 entries with more things weird than this");
+        sp.info->get_config("opt_max_weird_count",&opt_max_weird_count,"Number of 'weird' counts to ignore a FAT32 entry");
         sp.info->get_config("opt_last_year",&opt_last_year,"Ignore FAT32 entries with a later year than this");
 
         debug = sp.info->config->debug;
