@@ -25,6 +25,7 @@
  | "IMAGE_FILE_MACHINE_IA64"
  @NumberOfSections     => %d
  @TimeDateStamp        => %d
+ @TimeDateStampISO     => %s
  @PointToSymbolTable   => %d
  @NumberOfSymbols      => %d
  @SizeOfOptionalHeader => %d
@@ -337,8 +338,8 @@ struct  flagnames_t {
     const char *name;
 };
 
-static void decode_flags (stringstream &xml,
-                          const string &sectionName,
+static void decode_flags (std::stringstream &xml,
+                          const std::string &sectionName,
                           const struct flagnames_t flagnames[],
                           const uint32_t flags)
 {
@@ -351,7 +352,7 @@ static void decode_flags (stringstream &xml,
 
 // this is going to return a string because it returns empty string if
 // the value was not found
-static string match_switch_case (const struct flagnames_t flagnames[],
+static std::string match_switch_case (const struct flagnames_t flagnames[],
                                  const uint32_t needle)
 {
     int i;
@@ -494,7 +495,7 @@ static bool valid_section_name(const std::string &sectionName)
     return true;
 }
 
-static string scan_winpe_verify (const sbuf_t &sbuf)
+static std::string scan_winpe_verify (const sbuf_t &sbuf)
 {
     //const uint8_t * data = sbuf.buf;
     size_t size          = sbuf.bufsize;
@@ -503,7 +504,7 @@ static string scan_winpe_verify (const sbuf_t &sbuf)
     size_t ohw_offset                          = 0;    // OptionalHeaderWindows
     //uint32_t     header_offset;
     int          section_i;
-    stringstream xml;
+    std::stringstream xml;
     //int dlli;
     
     // set Pe_FileHeader to correct address
@@ -525,13 +526,14 @@ static string scan_winpe_verify (const sbuf_t &sbuf)
     uint16_t pe_Machine              = sbuf.get16u(header_offset);
     uint16_t pe_NumberOfSections     = sbuf.get16u(header_offset + 2);
     uint32_t pe_TimeDateStamp        = sbuf.get32u(header_offset + 4);
+    std::string pe_TimeDateStampISO  = unixTimeToISODate(pe_TimeDateStamp);
     uint32_t pe_PointerToSymbolTable = sbuf.get32u(header_offset + 8);
     uint32_t pe_NumberOfSymbols      = sbuf.get32u(header_offset + 12);
     uint16_t pe_SizeOfOptionalHeader = sbuf.get16u(header_offset + 16);
     uint16_t pe_Characteristics      = sbuf.get16u(header_offset + 18);
 
     // 2^11.75 confidence, 2^20.25 to go
-    string Machine = match_switch_case(pe_fileheader_machine, pe_Machine);
+    std::string Machine = match_switch_case(pe_fileheader_machine, pe_Machine);
     if (Machine == "") return "";
         
     // A PE with (0|>256) sections? Doubtful
@@ -545,6 +547,7 @@ static string scan_winpe_verify (const sbuf_t &sbuf)
     xml << " Machine=\""              << Machine                  << "\"";
     xml << " NumberOfSections=\""     << pe_NumberOfSections     << "\"";
     xml << " TimeDateStamp=\""        << pe_TimeDateStamp        << "\"";
+    xml << " TimeDateStampISO=\""     << pe_TimeDateStampISO     << "\"";
     xml << " PointerToSymbolTable=\"" << pe_PointerToSymbolTable << "\"";
     xml << " NumberOfSymbols=\""      << pe_NumberOfSymbols      << "\"";
     xml << " SizeOfOptionalHeader=\"" << pe_SizeOfOptionalHeader << "\"";
@@ -606,8 +609,8 @@ static string scan_winpe_verify (const sbuf_t &sbuf)
         xml << " SizeOfCode=\""              << pe_SizeOfCode                 << "\"";
         xml << " SizeOfInitializedData=\""   << pe_SizeOfInitializedData      << "\"";
         xml << " SizeOfUninitializedData=\"" << pe_SizeOfUninitializedData    << "\"";
-        xml << " AddressOfEntryPoint=\"0x"   << hex << pe_AddressOfEntryPoint << "\"";
-        xml << " BaseOfCode=\"0x"            << hex << pe_BaseOfCode          << "\"";
+        xml << " AddressOfEntryPoint=\"0x"   << std::hex << pe_AddressOfEntryPoint << "\"";
+        xml << " BaseOfCode=\"0x"            << std::hex << pe_BaseOfCode          << "\"";
         
         break;
 
@@ -629,8 +632,8 @@ static string scan_winpe_verify (const sbuf_t &sbuf)
         xml << " SizeOfCode=\""              << pe_SizeOfCode                 << "\"";
         xml << " SizeOfInitializedData=\""   << pe_SizeOfInitializedData      << "\"";
         xml << " SizeOfUninitializedData=\"" << pe_SizeOfUninitializedData    << "\"";
-        xml << " AddressOfEntryPoint=\"0x"   << hex << pe_AddressOfEntryPoint << "\"";
-        xml << " BaseOfCode=\"0x"            << hex << pe_BaseOfCode          << "\"";
+        xml << " AddressOfEntryPoint=\"0x"   << std::hex << pe_AddressOfEntryPoint << "\"";
+        xml << " BaseOfCode=\"0x"            << std::hex << pe_BaseOfCode          << "\"";
         
         break;
         
@@ -672,7 +675,7 @@ static string scan_winpe_verify (const sbuf_t &sbuf)
     uint64_t pe_SizeOfHeapReserve = 0;
     uint64_t pe_SizeOfHeapCommit = 0;
     
-    string Subsystem = "";
+    std::string Subsystem = "";
     bool ohw_xml = true;
     
     if (    (pe_Magic == IMAGE_FILE_TYPE_PE32)
@@ -749,7 +752,7 @@ static string scan_winpe_verify (const sbuf_t &sbuf)
     
     if (ohw_xml) {
         xml << "<OptionalHeaderWindows";
-        xml << " ImageBase=\"0x"                 << hex << pe_ImageBase            << "\"";
+        xml << " ImageBase=\"0x"                 << std::hex << pe_ImageBase       << "\"";
         xml << " SectionAlignment=\""            << pe_SectionAlignment            << "\"";
         xml << " FileAlignment=\""               << pe_FileAlignment               << "\"";
         xml << " MajorOperatingSystemVersion=\"" << pe_MajorOperatingSystemVersion << "\"";
@@ -761,8 +764,8 @@ static string scan_winpe_verify (const sbuf_t &sbuf)
         xml << " Win32VersionValue=\""           << pe_Win32VersionValue           << "\"";
         xml << " SizeOfImage=\""                 << pe_SizeOfImage                 << "\"";
         xml << " SizeOfHeaders=\""               << pe_SizeOfHeaders               << "\"";
-        xml << " CheckSum=\"0x"                  << hex << pe_CheckSum             << "\"";
-        xml << " SubSystem=\""                   << Subsystem                         << "\"";
+        xml << " CheckSum=\"0x"                  << std::hex << pe_CheckSum        << "\"";
+        xml << " SubSystem=\""                   << Subsystem                      << "\"";
         xml << " SizeOfStackReserve=\""          << pe_SizeOfStackReserve          << "\"";
         xml << " SizeOfStackCommit=\""           << pe_SizeOfStackCommit           << "\"";
         xml << " SizeOfHeapReserve=\""           << pe_SizeOfHeapReserve           << "\"";
@@ -940,13 +943,12 @@ static string scan_winpe_verify (const sbuf_t &sbuf)
     return xml.str();
 }
 
-static be13::hash_def hasher;
 extern "C"
 void scan_winpe (const class scanner_params &sp,
 		 const recursion_control_block &rcb)
 {
     assert(sp.sp_version==scanner_params::CURRENT_SP_VERSION);
-    string xml;
+    std::string xml;
     
     if (sp.phase == scanner_params::PHASE_STARTUP){
         assert(sp.info->si_version==scanner_info::CURRENT_SI_VERSION);
@@ -955,8 +957,6 @@ void scan_winpe (const class scanner_params &sp,
         sp.info->scanner_version = "1.0.0";
         sp.info->feature_names.insert("winpe");
 
-        hasher    = sp.info->config->hasher;
-
         return;
     }
     
@@ -1000,7 +1000,7 @@ void scan_winpe (const class scanner_params &sp,
 		if (xml != "") {
 		    // If we have 4096 bytes, generate md5 hash
                     sbuf_t first4k(data,0,4096);
-		    std::string hash = hasher.func(first4k.buf,first4k.bufsize);
+		    std::string hash = f->fs.hasher.func(first4k.buf,first4k.bufsize);
 		    f->write(data.pos0,hash,xml);
 		}
 	    }
