@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # coding=UTF-8
 """
 Regression system: 
@@ -12,7 +12,7 @@ Options include:
  - Total number of features are reported and compared with the archives.
 """
 
-__version__ = "1.3.0"
+__version__ = "1.5.0"
 
 b'This module needs Python 2.7 or later.'
 
@@ -34,7 +34,10 @@ corp_default = "/corp"
 default_infile   = "nps-2009-ubnist1/ubnist1.gen3.raw"
 fast_infile      = "nps-2010-emails/nps-2010-emails.raw"
 full_infile      = "nps-2009-domexusers/nps-2009-domexusers.raw"
-exe = "src/bulk_extractor"
+exe              = "src/bulk_extractor"
+if not os.path.exists(exe):
+    exe = "../src/bulk_extractor"
+
 nps_drives_path = "/nps/drives/"
 BOM = codecs.BOM_UTF8.decode('utf-8')
 
@@ -84,7 +87,27 @@ answers = {"ubnist1.gen3":{"ALERTS_found.txt":88,
                               "zip.txt":240}
            }
 
+def get_scanners(exe):
+    scanners = set()
+    for line in Popen([exe,'-h'],stdout=PIPE).communicate()[0].decode('utf-8').split("\n"):
+        atoms = line.strip().split(" ")
+        if len(atoms)>0 and atoms[0] in ['-x','-e']:
+            scanners.add(atoms[1])
+    return scanners
+    
 
+def ptime(t):
+    r = ""
+    if t>3600:
+        h = t/3600
+        r = "%d hour " % h
+        t = t%3600
+    if t>60:
+        m = t / 60
+        r += "%d min " % m
+        t = t%60
+    r += "%d sec " % t
+    return r
 
 def find_file(fn):
     if os.path.exists(fn): return fn
@@ -225,19 +248,18 @@ def analyze_reportxml(xmldoc):
                 name,calls,seconds,seconds/calls,100.0*seconds/total))
     
     
-
 def analyze_outdir(outdir):
     """Print statistics about an output directory"""
     print("Analyze {}".format(outdir))
 
     b = bulk_extractor_reader.BulkReport(outdir)
     print("bulk_extractor version: {}".format(b.version()))
-    print("Filename:               {}".format(b.imagefile()))
+    print("Image filename:         {}".format(b.image_filename()))
     
     # Print which scanners were run and how long they took
     analyze_reportxml(b.xmldoc)
     
-    hfns = list(b.histograms())
+    hfns = list(b.histogram_files()) # histogram files
     print("")
     print("Histogram Files:        {}".format(len(hfns)))
 
@@ -255,17 +277,60 @@ def analyze_outdir(outdir):
             firstline = firstline.decode('utf-8')
         print("  {:>25} entries: {:>10,}  (top: {})".format(fn,len(h),firstline))
 
-    fnpart = ".".join(b.imagefile().split('/')[-1].split('.')[:-1])
-    ffns = list(b.feature_files())
-    print("")
-    print("Feature Files:        {}".format(len(ffns)))
-    for fn in sorted(ffns):
-        lines = 0
-        for line in b.open(fn,'rb'):
-            if not bulk_extractor_reader.is_comment_line(line):
-                lines += 1
-        print("  {:>25} features: {:>12,}  {}".format(fn,lines,analyze_warning(fnpart,fn,lines)))
-    
+    fnpart = ".".join(b.image_filename().split('/')[-1].split('.')[:-1])
+
+    ffns = sorted(list(b.feature_files())) 
+    if ffns:
+        features = {}
+        print("")
+        print("Feature Files:        {}".format(len(ffns)))
+        for fn in ffns:     # feature files
+            lines = 0
+            for line in b.open(fn,'r'):
+                if not bulk_extractor_reader.is_comment_line(line):
+                    lines += 1
+                    features[fn] = lines
+            print("  {:>25} features: {:>12,}  {}".format(fn,lines,analyze_warning(fnpart,fn,lines)))
+                    
+        # If there is a SQLite database, analyze that too!
+    if args.featurefile and args.featuresql:
+        import sqlite3
+        conn = sqlite3.connect(os.path.join(outdir,"report.sqlite"))
+        if conn:
+            c = conn.cursor()
+            c.execute("PRAGMA cache_size = 200000")
+            print("Comparing SQLite3 database to feature files:")
+            for fn in ffns:
+                try:
+                    table = "f_"+fn.lower().replace(".txt","")
+                    cmd = "select count(*) from "+table
+                    print(cmd)
+                    c.execute(cmd);
+                    ct = c.fetchone()[0]
+                    print("{}:   {}  {}".format(fn,features[fn],ct))
+                    # Now check them all to make sure that the all match
+                    count = 0
+                    for line in b.open(fn,'r'):
+                        ary = bulk_extractor_reader.parse_feature_line(line)
+                        if ary:
+                            (path,feature) = ary[0:2]
+                            path = path.decode('utf-8')
+                            feature = feature.decode('utf-8')
+                            c.execute("select count(*) from "+table+" where path=? and feature_eutf8=?",(path,feature))
+                            ct = c.fetchone()[0]
+                            if ct==1:
+                                #print("feature {} {} in table {} ({})".format(path,feature,table,ct))
+                                pass
+                            if ct==0:
+                                #pass
+                                print("feature {} {} not in table {} ({})".format(path,feature,table,ct))
+                            count += 1
+                            if count>args.featuretest: 
+                                break
+
+                except sqlite3.OperationalError as e:
+                    print(e)
+
 
 def make_zip(dname):
     archive_name = dname+".zip"
@@ -287,30 +352,46 @@ def make_outdir(outdir_base):
 
 def run(cmd):
     print(" ".join(cmd))
+    if args.dry_run: return
     r = call(cmd)
     if r!=0:
         raise RuntimeError("{} crashed with error code {}".format(args.exe,r))
 
-def run_outdir(outdir,gdb=False):
+def run_outdir(gdb=False):
+    """Run bulk_extarctor to a given output directory """
+    outdir = make_outdir(args.outdir)
     print("run_outdir: ",outdir)
     cargs=['-o',outdir]
+    if args.featuresql:
+        cargs += ['-S','write_feature_sqlite3=YES']
+    else:
+        cargs += ['-S','write_feature_sqlite3=NO']
+    if args.featurefile:
+        cargs += ['-S','write_feature_files=YES']
+    else:
+        cargs += ['-S','write_feature_files=NO']
     if args.jobs: cargs += ['-j'+str(args.jobs)]
     if args.pagesize: cargs += ['-G'+str(args.pagesize)]
     if args.marginsize: cargs += ['-g'+str(args.marginsize)]
     
-    cargs += ['-e','all']    # enable all scanners
-    #cargs += ['-e','wordlist']    # enable all scanners
+    cargs += ['-e','all']    # enable 'all' scanners 
     if args.extra:
+        while "  " in args.extra:
+            args.extra = args.extra.replace("  "," ")
         cargs += args.extra.split(" ")
     if args.debug: cargs += ['-d'+str(args.debug)]
 
-    if not args.nofind:
+    if args.find:
         cargs += ['-r','tests/alert_list.txt']
         cargs += ['-w','tests/stop_list.txt']
         cargs += ['-w','tests/stop_list_context.txt']
         cargs += ['-f','[a-z\.0-9]*@gsa.gov']
         cargs += ['-F','tests/find_list.txt']
-    cargs += [args.image]
+
+    if args.image:
+        cargs += [args.image]
+    else:
+        cargs += ['-R',args.datadir]
 
     # Now that we have a command, figure out how to run it...
     if gdb:
@@ -324,6 +405,7 @@ def run_outdir(outdir,gdb=False):
     else:
         cmd = [args.exe] + cargs
     run(cmd)
+    return outdir
 
              
 def sort_outdir(outdir):
@@ -336,13 +418,10 @@ def sort_outdir(outdir):
         fns  = fn+".sorted"
         os.environ['LC_ALL']='C' # make sure we sort in C order
         call(['sort','--buffer-size=4000000000',fn],stdout=open(fns,"w"))
+        # count how many lines
         wcout = Popen(['wc','-l',fns],stdout=PIPE).communicate()[0].decode('utf-8')
         lines = int(wcout.strip().split(" ")[0])
-        if lines>0:
-            call(['tail','-1',fns],stdout=open(fn,"w")) # copy over the UTF-8 header
-            cmd = ['head','-'+str(lines-1),fns]
-            call(cmd,stdout=open(fn,"a"))
-        os.unlink(fns)
+        os.rename(fns,fn)
 
 def check(fn,lines):
     found_lines = len(file(fn).read().split("\n"))-1
@@ -377,9 +456,9 @@ def invalid_feature_file_line(line,fields):
     return None
     
 def validate_file(f,kind):
-    kml = "kml" in f.name
+    is_kml = f.name.endswith(".kml")
     linenumber = 0
-    print("Validate UTF-8 encoding in ",f.name)
+    if args.debug: print("Validate UTF-8 encoding in ",f.name)
     for lineb in f:
         linenumber += 1
         lineb = lineb[:-1]      # remove the \n
@@ -397,20 +476,21 @@ def validate_file(f,kind):
             r = invalid_feature_file_line(line,fields)
             if r:
                 print("{}: {:8} {} Invalid feature file line: {}".format(f.name,linenumber,r,line))
-            if kml and fields[1].count("kml")!=2:
+            if is_kml and fields[1].count("kml")!=2:
                 print("{}: {:8} Invalid KML line: {}".format(f.name,linenumber,line))
 
 
 
-def validate_report(fn):
+def validate_report(fn,do_validate=True):
     """Make sure all of the lines in all of the files in the outdir are UTF-8 and that
     the feature files have 3 or more fields on each line.
+    And validate the XML!
     """
     import glob,os.path
-    print("\nValidate Report: ",fn)
+    if args.debug: print("Validating Report: ",fn)
     res = {}
     if os.path.isdir(fn) or fn.endswith(".zip"):
-        b = bulk_extractor_reader.BulkReport(fn)
+        b = bulk_extractor_reader.BulkReport(fn,do_validate=True)
         for fn in b.feature_files():
             if os.path.basename(fn) in str(args.ignore):
                 print("** ignore {} **".format(fn))
@@ -426,14 +506,20 @@ def identify_filenames(outdir):
     ifname    = os.path.dirname(os.path.realpath(__file__)) + "/../python/identify_filenames.py"
     res = call([sys.executable,ifname,outdir,if_outdir,'--all'])
 
-
-
 def diff(dname1,dname2):
     args.max = int(args.max)
+    def safe_filesize(fn):
+        try:
+            return os.path.getsize(fn)
+        except FileNotFoundError:
+            return 0
+
     def files_in_dir(dname):
-        return [fn.replace(dname+"/","") for fn in glob.glob(dname+"/*")]
+        files = [fn.replace(dname+"/","") for fn in glob.glob(dname+"/*") if os.path.isfile(fn)]
+        return filter(lambda fn: not fn.endswith(".sqlite") ,files)
     def lines_to_set(fn):
-        return set(open(fn).read().split("\n"))
+        "Read the lines in a file and return them as a set. Ignore the lines beginning with #"
+        return set(filter(lambda line:line[0:1]!='#',open(fn).read().split("\n")))
     
     files1 = set(files_in_dir(dname1))
     files2 = set(files_in_dir(dname2))
@@ -442,73 +528,152 @@ def diff(dname1,dname2):
     if files2.difference(files1):
         print("Files only in {}:\n   {}".format(dname2," ".join(files2.difference(files1))))
 
-    # Look at the common files
+    # Look at the common files. For each report the files only in one or the other
     common = files1.intersection(files2)
-    for fn in common:
+    for fn in sorted(common):
+        fn1 = os.path.join(dname1,fn)
+        fn2 = os.path.join(dname2,fn)
+        
+        if fn=="report.xml":
+            continue
+        if fn=="wordlist.txt" and not args.diffwordlist:
+            continue
+        if fn=="packets.pcap": 
+            s1 = os.path.getsize(fn1)
+            s2 = os.path.getsize(fn2)
+            if s1!=s2:
+                print("size({})={} but size({})={}\n",fn1,s1,fn2,s2)
+            continue
         def print_diff(dname,prefix,diffset):
             if not diffset:
                 return
-            print("Only in {}: {} lines".format(dname,len(diffset)))
+            print("\nOnly in {}: {} lines".format(os.path.join(dname,fn),len(diffset)))
             count = 0
             for line in sorted(diffset):
-                print("{}{}".format(prefix,line))
+                extra = ""
+                if len(line) > args.diffwidth: extra="..."
+                print("{}{}{}".format(prefix,line[0:args.diffwidth],extra))
                 count += 1
                 if count>args.max:
                     print(" ... +{} more lines".format(len(diffset)-int(args.max)))
-                    return
+                    break
+            return len(diffset)
 
-        print("regressdiff {} {}:".format(os.path.join(dname1,fn),os.path.join(dname2,fn)))
-        lines1 = lines_to_set(os.path.join(dname1,fn))
-        lines2 = lines_to_set(os.path.join(dname2,fn))
-        print_diff(dname1,"<",lines1.difference(lines2))
-        print_diff(dname2,">",lines2.difference(lines1))
-        if lines1 != lines2:
-            print("")
+        lines1 = lines_to_set(fn1)
+        lines2 = lines_to_set(fn2)
+        if lines1!=lines2:
+            print("regressdiff {}:".format(fn))
+            count1 = print_diff(dname1,"<",lines1.difference(lines2))
+            count2 = print_diff(dname2,">",lines2.difference(lines1))
+            if count1 or count2:
+                print("\n-------------\n")
 
 
 def run_and_analyze():
     global args
-    def ptime(t):
-        r = ""
-        if t>3600:
-            h = t/3600
-            r = "%d hour " % h
-            t = t%3600
-        if t>60:
-            m = t / 60
-            r += "%d min " % m
-            t = t%60
-        r += "%d sec " % t
-        return r
-    outdir = make_outdir(args.outdir)
     t0 = time.time()
-    run_outdir(outdir,args.gdb)
+    outdir = run_outdir(args.gdb)
     sort_outdir(outdir)
     validate_report(outdir)
-    if_outdir = identify_filenames(outdir)
+    if args.identify_filenames:
+        if_outdir = identify_filenames(outdir)
     analyze_outdir(outdir)
-    print("Regression finished at {}. Elapsed time: {} Output in {}".format(time.asctime(),ptime(time.time()-t0),outdir))
+    t = time.time() - t0
+    print("Regression finished at {}. Elapsed time: {} ({} sec)\nOutput in {}".format(
+        time.asctime(),ptime(t),t,outdir))
 
+def datadircomp(dir1,dir2):
+    print("Validating reports in {} and {}".format(dir1,dir2))
+    for d in [dir1,dir2]:
+        for fn in glob.glob(os.path.join(d,"*")):
+            validate_report(fn,do_validate=True)
+    print("Comparing outputs".format(dir1,dir2))
+    for fn1 in glob.glob(os.path.join(dir1,"*")):
+        fn2 = fn1.replace(dir1,dir2)
+        diff(fn1,fn2)
+    exit(0)
+
+def datacheck():
+    # First run bulk_extractor on
+    args.image = None
+    args.datadir = 'Data'
+    args.find  = None
+    if args.extra:
+        args.extra += " "
+    else:
+        args.extra = ""
+    if "outlook" in get_scanners(args.exe):
+        args.extra += "-e outlook"
+    outdir = run_outdir()
+    datacheckreport(outdir)
+        
+def datacheckreport(outdir):
+    # Open the datadir
+    print("opening ",outdir)
+    b = bulk_extractor_reader.BulkReport(outdir)
+    found_features = {}
+    print("Feature files:",list(b.feature_files()))
+    print("Histogram files:",list(b.histogram_files()))
+    for fn in b.feature_files():
+        print("Reading feature file {}".format(fn))
+        for (pos,feature,context) in b.read_features(fn):
+            found_features[pos] = feature
+    print("Now reading features from data_check.txt")
+    not_found = {}
+    report_mismatches = False 
+    found_count = 0
+    for line in open("data_check.txt","rb"):
+        y = bulk_extractor_reader.parse_feature_line(line)
+        if y:
+            (pos,feature,context) = y
+            if pos in found_features:
+                found_count += 1
+                #print("{} found".format(pos.decode('utf-8')))
+                if found_features[pos]!=feature and report_mismatches:
+                    if found_features[pos]!=b'<CACHED>' and feature!=b'<CACHED>':
+                        print("   {} != {}".format(feature,found_features[pos]))
+            else:
+                not_found[pos] = feature
+    for pos in sorted(not_found):
+        print("{} not found {}".format(pos,not_found[pos]))
+    print("Total features found: {}".format(found_count))
+    print("Total features not found: {}".format(len(not_found)))
+        
+        
 if __name__=="__main__":
     import argparse 
     global args
     import sys,time
 
-    parser = argparse.ArgumentParser(description="Perform regression testing on bulk_extractor")
+    parser = argparse.ArgumentParser(description="Perform regression testing on bulk_extractor",
+                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
     parser.add_argument("--gdb",help="run under gdb",action="store_true")
     parser.add_argument("--debug",help="debug level",type=int)
-    parser.add_argument("--outdir",help="output directory base",default="regress")
+    parser.add_argument("--outdir",help="output directory base",default=None)
     parser.add_argument("--exe",help="Executable to run (default {})".format(exe),default=exe)
     parser.add_argument("--image",help="image to scan (default {})".format(default_infile),default=default_infile)
     parser.add_argument("--fast",help="Run with "+fast_infile,action="store_true")
     parser.add_argument("--full",help="Run with "+full_infile,action="store_true")
     parser.add_argument("--jobs",help="Specifies number of worker threads",type=int)
     parser.add_argument("--pagesize",help="Specifies page size",type=int)
-    parser.add_argument("--nofind",help="Does not do find test (faster)",action="store_true")
+    parser.add_argument("--no-find",dest='find',help="Does not do find test (faster)",action="store_false")
+
+    g = parser.add_mutually_exclusive_group()
+    g.add_argument("--featuresql",dest='featuresql',action='store_true',help="Enable SQL feature files")
+    g.add_argument("--no-featuresql",dest='featuresql',action='store_false',help="Disable SQL feature files")
+    parser.set_defaults(featuresql=True)
+
+    g = parser.add_mutually_exclusive_group()
+    g.add_argument("--featurefile",dest='featurefile',action='store_true',help="Enable FILE feature files")
+    g.add_argument("--no-featurefile",dest='featurefile',action='store_false',help="Disable FILE feature files")
+    parser.set_defaults(featurefile=True)
+
     parser.add_argument("--marginsize",help="Specifies the margin size",type=int)
     parser.add_argument("--extra",help="Specify extra arguments")
     parser.add_argument("--gprof",help="Recompile and run with gprof",action="store_true")
-    parser.add_argument("--diff",help="diff mode. Compare two outputs",type=str,nargs='*')
+    parser.add_argument("--diff",help="diff mode. Compare two reports",type=str,nargs='*')
+    parser.add_argument("--diffwidth",type=int,help="Number of characters to display on diff lines",default=170)
+    parser.add_argument("--diffwordlist",type=bool,help="compare wordlist.txt",default=False)
     parser.add_argument("--max",help="Maximum number of differences to display",default="5")
     parser.add_argument("--memdebug",help="Look for memory errors",action="store_true")
     parser.add_argument("--analyze",help="Analyze a bulk_extractor output directory")
@@ -517,16 +682,27 @@ if __name__=="__main__":
     parser.add_argument("--validate",help="Validate the contents of a report (do not run bulk_extractor)",
                         type=str,nargs='*')
     parser.add_argument("--ignore",help="Specifies a feature file or files (file1,file2) to ignore")
-    parser.add_argument("--sort",help="Sort the feature files",type=str,nargs='*')
     parser.add_argument("--reproduce",help="specifies a bulk_extractor output "
             + "file from a crash and produces bulk_extractor flags to quickly "
             + "reproduce the crash")
     parser.add_argument("--clearcache",help="clear the disk cache",action="store_true")
     parser.add_argument("--tune",help="run bulk_extractor tuning. Args are coded in this script.",action="store_true")
+    parser.add_argument("--featuretest",default=50,type=int,
+                        help="Specifies number of features to test to make sure they are in the SQL database",)
+    parser.add_argument("--no-identify_filenames",dest='identify_filenames',action="store_false",
+                        help="Do not run identify_filenames")
+    parser.add_argument("--datadir",help="Process all files in a directory")
+    parser.add_argument("--dry-run",help="Don't actually run the program",action='store_true')
+    parser.add_argument("--datadircomp",help="Compare two data dirs")
+    parser.add_argument("--datacheck",help="Runs BE on the files in Data/ directory and makes sure that all of the features in data_check.txt are found",action='store_true')
+    parser.add_argument("--datacheckreport",help="Checks the files in in Data/ directory and makes sure that all of the features in data_check.txt are found")
 
     args = parser.parse_args()
+    
+    if args.datadircomp:
+        dirs = args.datadircomp.split(",")
+        datadircomp(dirs[0],dirs[1])
 
-    # these are mostly for testing
     if args.validate:
         for v in args.validate:
             try:
@@ -562,8 +738,38 @@ if __name__=="__main__":
     if not os.path.exists(args.exe):
         raise RuntimeError("{} does not exist".format(args.exe))
 
+    if args.clearcache: clear_cache()
+
+    ################################################################
+    ### After this point the program is run
+    ################################################################
+
+    # Are we processing an entire directory?
+    if args.datadir:
+        try:
+            os.mkdir(args.outdir)
+        except FileExistsError:
+            pass
+        if not args.outdir:
+            print("--datadir requires that outdir be specified")
+            exit(0)
+        for fn in glob.glob(os.path.join(args.datadir,"*")):
+            args.image  = fn
+            args.outdir = os.path.join(args.outdir,os.path.basename(fn))
+            run_outdir()
+        exit(0)
+
     # Find the bulk_extractor version and add it to the outdir
+    if args.outdir==None:
+        args.outdir="regress"
+
     args.outdir += "-"+bulk_extractor_reader.be_version(args.exe)
+
+    if args.datacheck: datacheck() ; exit(1)
+
+    if args.datacheckreport: datacheckreport(args.datacheckreport); exit(1)
+
+
     corp = os.getenv("DOMEX_CORP")
     if not corp:
         corp = corp_default
@@ -622,10 +828,6 @@ if __name__=="__main__":
             raise ValueError("--diff requires two arguments")
         diff(args.diff[0],args.diff[1])
         exit(0)
-    if args.sort:
-        for s in args.sort:
-            sort_outdir(s)
-        exit(0)
 
     if args.tune:
         print("tune_jobs_start: ",tune_jobs_start)
@@ -650,9 +852,8 @@ if __name__=="__main__":
             args.pagesize = pagesize
             args.margin = margin
             clear_cache()
-            outdir = make_outdir(args.outdir+"-{}-{}-{}".format(jobs,pagesize,margin))
+            args.outdir = make_outdir(args.outdir+"-{}-{}-{}".format(jobs,pagesize,margin))
             clear_cache()
-            run_outdir(outdir)
             ofn = "report-{}-{}-{}.xml".format(j,p,m)
             os.rename(outdir+"/report.xml",ofn)
 
@@ -665,8 +866,5 @@ if __name__=="__main__":
             
         exit(0)
         
-    if args.clearcache:
-        clear_cache()
-
     run_and_analyze()
 
